package com.qmth.sop.business.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.sop.business.bean.params.DingAddParam;
import com.qmth.sop.business.bean.result.CanChooseSopResult;
import com.qmth.sop.business.entity.TBDingAdd;
import com.qmth.sop.business.mapper.TBDingAddMapper;
import com.qmth.sop.business.service.TBCrmDetailService;
import com.qmth.sop.business.service.TBDingAddService;
import com.qmth.sop.business.service.TBUserArchivesAllocationService;
import com.qmth.sop.business.service.TBUserArchivesService;
import com.qmth.sop.common.enums.ExceptionResultEnum;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.Objects;

/**
 * <p>
 * 工时追加表 服务实现类
 * </p>
 *
 * @author CaoZixuan
 * @since 2024-11-04
 */
@Service
public class TBDingAddServiceImpl extends ServiceImpl<TBDingAddMapper, TBDingAdd> implements TBDingAddService {

    @Resource
    private TBUserArchivesService tbUserArchivesService;

    @Resource
    private TBUserArchivesAllocationService tbUserArchivesAllocationService;

    @Resource
    private TBCrmDetailService tbCrmDetailService;

    @Override
    public CanChooseSopResult findCanChooseSopResultList(Long serviceId, Long userId) {
        CanChooseSopResult result = new CanChooseSopResult();
        boolean containsRegionCoordinator = tbUserArchivesService.containsRegionCoordinator(userId);
        if (containsRegionCoordinator) {
            result.setNeedChoose(false);
        } else {
            result.setNeedChoose(true);
            result.setCrmDetailInfo(this.baseMapper.findSopInfoByServiceIdAndUserId(serviceId, userId));
        }
        return result;
    }

    @Transactional
    @Override
    public void dingAddSubmit(DingAddParam dingAddParam, Long requestUserId) {
        Long userId = dingAddParam.getUserId();
        Long serviceId = dingAddParam.getServiceId();
        String sopNo = dingAddParam.getSopNo();
        BigDecimal addDays = dingAddParam.getAddDays();
        BigDecimal addHours = dingAddParam.getAddHours();

        boolean isCoordinator = tbUserArchivesService.containsRegionCoordinator(userId);
        if (!isCoordinator && Objects.isNull(sopNo)) {
            throw ExceptionResultEnum.ERROR.exception("非区域协调员必须选择SOP");
        }

        TBDingAdd dingAdd = new TBDingAdd();
        dingAdd.setServiceId(serviceId);
        dingAdd.setUserId(userId);
        if (Objects.nonNull(sopNo)) {
            dingAdd.setSopNo(sopNo);
        }
        dingAdd.setAddDays(addDays);
        dingAdd.setAddHours(addHours);
        dingAdd.insertInfo(requestUserId);

        this.remove(new QueryWrapper<TBDingAdd>().lambda().eq(TBDingAdd::getServiceId, serviceId).eq(TBDingAdd::getUserId, userId));
        this.save(dingAdd);
    }
}
